//
//  SoraMoyoPlugin.m
//  SoraMoyoPlugin
//
//  Created by Cyan K on 05/11/24.
//  cyanworks.net
//

#import "SoraMoyoPlugin.h"

@implementation SoraMoyoPlugin

#pragma mark Initialization and deallocation
- (id)initWithWebView:(WebView *)webView
{
    if (self = [super init]) {
		kishocho = [[SMKishocho alloc] init];

		// Add observer: Default Notification Center
		[[NSNotificationCenter defaultCenter]
			addObserver:self selector:@selector(weatherDownloadFinished:)
			name:@"weatherDownloadFinished" object:kishocho];
		[[NSNotificationCenter defaultCenter]
			addObserver:self selector:@selector(connectionError:)
			name:@"connectionError" object:kishocho];
		[[NSNotificationCenter defaultCenter]
			addObserver:self selector:@selector(parseError:)
			name:@"parseError" object:kishocho];

		// Add observer: Shared NSWorkspace Notification
		[[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self
			selector:@selector(systemDidWake:)
			name:@"NSWorkspaceDidWakeNotification" object:nil];
	}
	return self;
}

- (void)dealloc
{
	// Remove observer from notification center
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];

	[kishocho release];
	[webScriptObject release];
	[super dealloc];
}

#pragma mark WebScripting
- (void)windowScriptObjectAvailable:(WebScriptObject *)windowScriptObject
{
	[windowScriptObject setValue:self forKey:@"SoraMoyoPlugin"];

	if (webScriptObject)
		[webScriptObject release];
	webScriptObject = [windowScriptObject retain];
}

+ (NSString *)webScriptNameForSelector:(SEL)aSelector
{
	if (aSelector == @selector(loadWeather:withMode:baseURI:ifModSince:))
		return @"loadWeather";
	return nil;
}

+ (BOOL)isSelectorExcludedFromWebScript:(SEL)aSelect
{	
	if (aSelect == @selector(loadWeather:withMode:baseURI:ifModSince:))
		return NO;
	if (aSelect == @selector(stopLoading))
		return NO;
	return YES;
}

#pragma mark Implementation
- (void)loadWeather:(NSString *)area withMode:(NSString *)mode
		baseURI:(NSString *)uri ifModSince:(NSString *)ims
{
	if ([mode isEqualToString:@"3days"])
		[kishocho loadWeather:area withMode:SM3daysWeather
			baseURI:[NSURL URLWithString:uri] ifModSince:ims];
	if ([mode isEqualToString:@"7days"])
		[kishocho loadWeather:area withMode:SM7daysWeather
			baseURI:[NSURL URLWithString:uri] ifModSince:ims];
}

- (void)stopLoading
{
	[kishocho stopLoading];
}

#pragma mark Notification
- (void)weatherDownloadFinished:(NSNotification *)aNotification
{
	#ifdef DEBUG
		NSLog(@"%@ is notified.", [aNotification name]);
	#endif
	switch ([kishocho httpStatus]) {
		case 200:
			[webScriptObject callWebScriptMethod:@"downloadFinished"
				withArguments:[NSArray arrayWithObject:[kishocho weatherData]]];
			break;
		case 304:
			[webScriptObject callWebScriptMethod:@"downloadNotModified" withArguments:nil];
			break;
		default:
			[webScriptObject callWebScriptMethod:@"downloadError"
				withArguments:[NSArray arrayWithObject:
					[NSString stringWithFormat:@"HTTP status %d", [kishocho httpStatus]]]];
			break;
	}
}

- (void)connectionError:(NSNotification *)aNotification
{
	#ifdef DEBUG
		NSLog(@"%@ is notified.", [aNotification name]);
	#endif
	[webScriptObject callWebScriptMethod:@"downloadError"
		withArguments:[NSArray arrayWithObject:@"Connection Error"]];
}

- (void)parseError:(NSNotification *)aNotification
{
	#ifdef DEBUG
		NSLog(@"%@ is notified.", [aNotification name]);
	#endif
	[webScriptObject callWebScriptMethod:@"downloadError"
		withArguments:[NSArray arrayWithObject:@"Parse Error"]];
}

// System waked up from the sleep mode
- (void)systemDidWake:(NSNotification*)aNotification
{
	#ifdef DEBUG
		NSLog(@"%@ is notified.", [aNotification name]);
	#endif
	[webScriptObject callWebScriptMethod:@"systemDidWake" withArguments:nil];
}

@end
